package de.hsbo.fbv.bmg.vectors;

/**
 * Basisklasse fr Ebenen und Dreiecke, behandelt bevorzugt 2-dim. und 3-dim. Vektoren
 * entsprechend den Algorithmen der analytischen Geometrie.
 * Intern angewendet wird die vektorielle Gleichung:
 * v(u) = s*(1-u)*(1-v) + e*u + f*v
 * @author Zimmermann, HS Bochum, 2009
 *
 */
public class Plane {

	/**
	 * Attribute Ortsvektoren fr Startpunkt und Endpunkte e und f 
	 */
	protected Vector s;		// Startpunkt-Vektor
	protected Vector e;		// Endpunkt-Vektor fr Richtung 1
	protected Vector f;		// Endpunkt-Vektor fr Richtung 2
	protected int dim = 0;	// Dimensionen der Punkt-Vektoren
	
	/**
	 * Konstruktor, bergibt Start- und Endpunkt
	 * @param sp - Startpunkt-Vektor
	 * @param ep - Endpunkt-Vektor 1
	 * @param fp - Endpunkt-Vektor 2
	 */
	public Plane(Vector sp, Vector ep, Vector fp) {
		try {
			if(sp.dim() == ep.dim() && sp.dim() == fp.dim()) {
				this.s = sp;
				this.e = ep;
				this.f = fp;
				this.dim = sp.dim();
			} else {
				throw new VectorDimensionException("dimensions differ");
			}
		} catch (VectorDimensionException e) {
			e.printStackTrace();
			this.s = null;
			this.e = null;
			this.f = null;
		}
	}
	
}
