package de.hsbo.fbv.bmg.vectors.aufgaben;

/**
 * Klasse fr Koordinaten, abgeleitet von Klasse Vector
 * @author zimmermann, HS Bochum
 *
 */
public class Coordinate extends Vector {
	
	// Parameter
	private static double EPS = 1.0e-4;	// Toleranz epsilon
	public static final int X = 0;		// Index der x-Komponente
	public static final int Y = 1;		// Index der y-Komponente
	public static final int Z = 2;		// Index der z-Komponente
	
	/**
	 * Konstruktor fr zweidimensionale Koordinaten
	 * @param x
	 * @param y
	 * @throws Exception
	 */
	public Coordinate(double x, double y) throws Exception {
		super(2);
		this.v[X] = x;
		this.v[Y] = y;
	}
	
	/**
	 * Konstruktor fr dreidimensionale Koordinaten
	 * @param x
	 * @param y
	 * @param z
	 * @throws Exception
	 */
	public Coordinate(double x, double y, double z) throws Exception {
		super(3);
		this.v[X] = x;
		this.v[Y] = y;
		this.v[Z] = z;
	}
	
	/**
	 * Erzeugt aus einem 2D- oder 3D-Vektor eine Koordinate.
	 * @param v
	 * @return
	 * @throws Exception
	 */
	public static Coordinate create(Vector v) throws Exception {
		if(v.v.length==2) {
			return new Coordinate(v.v[X], v.v[Y]);
		}
		else if(v.v.length==3) {
			return new Coordinate(v.v[X], v.v[Y], v.v[Z]);
		}
		else
			throw new Exception("Dimension must be equal to 2 or 3.");
	}
	
	/**
	 * Berechnet den Abstand zwischen zwei Koordinaten.
	 * @param b
	 * @return
	 * @throws Exception
	 */
	public double getDistance(Coordinate b) throws Exception {
		return (this.sub(b)).getLength();
	}
	
	/**
	 * Stellt fest, ob zwei Koordinaten die gleiche Lage besitzen.
	 * Vergleichskriterium ist der Toleranzwert EPS.
	 * @param b
	 * @return
	 * @throws Exception
	 */
	public boolean getEqual(Coordinate b) throws Exception {
		return ((this.getDistance(b))<EPS);
	}

	/**
	 * Gibt den Toleranzwert EPS an.
	 * @return
	 */
	public static double getEPS() {
		return EPS;
	}

	/**
	 * Setzt den Toleranzwert eps fest.
	 * @param eps
	 */
	public static void setEPS(double eps) {
		EPS = eps;
	}
	
	
}
