package de.hsbo.fbv.bmg.vectors.aufgaben;

import java.util.Arrays;

/**
 * Klasse fr Vektoren im n-dimensionalen Raum vom Typ double .
 * @author zimmermann, HS Bochum
 *
 */
public class Vector {
	
	// Attribute
	double[] v = null;
	
	// Konstruktoren
	/**
	 * Konstruktor fr einen n-dimensionalen Vektor; erzeugt einen Null-Vektor.
	 * @param n Dimension (darf nicht kleiner als 1 sein)
	 * @throws Exception 
	 */
	public Vector(int n) throws Exception {
		if(n>0)
			this.v = new double[n];
		else 
			throw new Exception("Dimension n less than 1.");
	}

	// Methoden
	/**
	 * Gibt die Komponente i aus.
	 * @param i
	 * @return
	 * @throws Exception
	 */
	public double getV(int i) throws Exception {
		if(i>=0 || i<v.length)
			return v[i];
		else 
			throw new Exception("Index i not allowed." );
	}

	/**
	 * berschreibt den Wert der Komponente i.
	 * @param i
	 * @param vi
	 * @throws Exception
	 */
	public void setV(int i, double vi) throws Exception {
		if(i>=0 || i<v.length)
			this.v[i] = vi;
		else 
			throw new Exception("Index i not allowed." );
	}
	
	/**
	 * Gibt die Dimension des Vektors aus.
	 * @return
	 */
	public int getDimension() {
		return this.v.length;
	}
	
	/**
	 * Addiert zum Vektor this den Vektor b (gleiche Dimension!).
	 * @param b zweiter Vektor 
	 * @return
	 * @throws Exception 
	 */
	public Vector add(Vector b) throws Exception {
		if(this.v.length==b.v.length) {
			Vector res = new Vector(v.length);
			for(int i=0; i<v.length; i++) {
				res.v[i] = this.v[i] + b.v[i];
			}
			return res;
		}
		else 
			throw new Exception("Dimensions must be equal.");
	}

	/**
	 * Subtrahiert vom Vektor this den Vektor b (gleiche Dimension!).
	 * @param b zweiter Vektor 
	 * @return
	 * @throws Exception 
	 */
	public Vector sub(Vector b) throws Exception {
		if(this.v.length==b.v.length) {
			Vector res = new Vector(v.length);
			for(int i=0; i<v.length; i++) {
				res.v[i] = this.v[i] - b.v[i];
			}
			return res;
		}
		else 
			throw new Exception("Dimensions must be equal.");
	}

	/**
	 * Multipliziert den Vektor this mit dem skalaren Wert b.
	 * @param b skalarer Wert 
	 * @return
	 * @throws Exception 
	 */
	public Vector mul(double b) throws Exception {
		Vector res = new Vector(v.length);
		for(int i=0; i<v.length; i++) {
			res.v[i] = this.v[i] * b;
		}
		return res;
	}
	
	/**
	 * Dividiert den Vektor this durch den skalaren Wert b.
	 * @param b skalarer Wert 
	 * @return
	 * @throws Exception 
	 */
	public Vector div(double b) throws Exception {
		Vector res = new Vector(v.length);
		for(int i=0; i<v.length; i++) {
			res.v[i] = this.v[i] / b;
		}
		return res;
	}

	/**
	 * Multipliziert den Vektor this mit dem Vektor b skalar (gleiche Dimension!).
	 * @param b zweiter Vektor 
	 * @return
	 * @throws Exception 
	 */
	public double mul(Vector b) throws Exception {
		if(this.v.length==b.v.length) {
			double res = 0;
			for(int i=0; i<v.length; i++) {
				res = res + this.v[i] * b.v[i];
			}
			return res;
		}
		else 
			throw new Exception("Dimensions must be equal.");
	}
	
	/**
	 * Berechnet die Lnge des Vektors.
	 * @return
	 * @throws Exception
	 */
	public double getLength() throws Exception {
		return Math.sqrt(this.mul(this)); 
	}
	
	/**
	 * Berechnet den normalisierten Vektor (Vektor gleicher Richtung mit Lnge 1.0).
	 * @return
	 * @throws Exception
	 */
	public Vector getNormalized() throws Exception {
		return this.div(this.getLength());
	}
	
	/**
	 * Berechnet das Kreuzprodukt fr 3-dim. Vektoren.
	 * @param b
	 * @return
	 * @throws Exception
	 */
	public Vector xmul3D(Vector b) throws Exception {
		if(this.v.length==3 && b.v.length==3) {
			Vector res = new Vector(3);
			res.v[0] =  this.v[1]*b.v[2] - this.v[2]*b.v[1];
			res.v[1] = -this.v[0]*b.v[2] + this.v[2]*b.v[0];
			res.v[2] =  this.v[0]*b.v[1] - this.v[1]*b.v[0];
			return res;
		}
		else 
			throw new Exception("Dimensions must be equal to 3.");
	}

	/**
	 * Berechnet das Kreuzprodukt fr 2-dim. Vektoren.
	 * @param b
	 * @return
	 * @throws Exception
	 */
	public double xmul2D(Vector b) throws Exception {
		if(this.v.length==2 && b.v.length==2) {
			double res =  this.v[0]*b.v[1] - this.v[1]*b.v[0];
			return res;
		}
		else 
			throw new Exception("Dimensions must be equal to 2.");
	}

	/**
	 * Berechnet den senkrechten Vektor (gleiche Lnge) auf einen 2-dim. Vektor.
	 * @return
	 * @throws Exception
	 */
	public Vector ortho2D() throws Exception {
		if(this.v.length==2) {
			Vector res = new Vector(2);
			res.v[0] = -this.v[1];
			res.v[1] =  this.v[0];
			return res;
		}
		else 
			throw new Exception("Dimension must be equal to 2.");
	}

	public String toString() {
		return Arrays.toString(this.v);
	}


}	// Ende Klasse
