package de.hsbo.fbv.bmg.vectors.demos;

import javax.vecmath.*;

/**
 * Programm zur Demonstration der Klassenbibliothek vecmath.jar<br>
 * Neben benutzerdefinierten Klassenbilbliotheken stellt auch die JDK-Umgebung  
 * eine grundlegende Klassenbibliothek (vecmath.jar) fr Vektoren und Matrizen 
 * bereit. Sie enthlt fr 2D-, 3D- und 4D-Anwendungen und verschiedene 
 * Datentypen (byte, int, float, double) angepasste Klassen von Vektoren 
 * und Matrizen, die in andere Java-Bilbliotheken wie z.B. Java3D 
 * und GeoTools eingebunden sind. <br>
 * Die Klassenhierarchie sieht fr alle Dimensionen und Datentypen gleich 
 * aus:<br>
 * Zu jeder abstrakten Elternklasse TupleNT (N steht fr 2=2D, 3=3D, 4=4D; T 
 * steht fr b=byte, i=int, f=float, d=double) gibt es je nach Anwendungsmglichkeit 
 * mehrere konkrete Kindklassen.<br> 
 * * VectorNT stellt zustzlich zu den Operationen add, sub und scale der Klasse 
 * TupleNT die Operationen fr Skalar- und Kreuzmultiplikationen bereit.<br>
 * * PointNT verkrpert Koordinaten der Form N und T.<br>
 * * Je nach Verwendung gibt es fr spezielle Kombinationen weitere Kindklassen, 
 * so z.B. Klassen fr Farbmodelle (Color3b, Color4b, Color3f, Color4f), die in 
 * der Computergraphik einsetzbar sind.<br>
 * <br>
 * Das folgende Programm demonstriert die Verwendung der Klassen von Tuple3d, womit 
 * dreidimensionale Vektoren vom Datentyp double verarbeitet werden.
 * @author A.Zimmermann, HS Bochum, 2012
 */
public class DemoVecMath {

	/**
	 * Methoden main()
	 * @param args
	 */
	public static void main(String[] args) {
		
		// Vektoren a und b deklarieren
		Tuple3d a = new Vector3d(10,10,10);
		System.out.println("Vektor a = "+a);
		Tuple3d b = new Vector3d(-10,10,10);
		System.out.println("Vector b = "+b);
		
		// Methoden der abstrakten Elternklasse Tuple3d
		
		// Sind die Vekotren a und b gleich?
		double t = 0.1;
		boolean equ = a.epsilonEquals(b, t);
		System.out.println("Ergebnis a==b: "+ equ + " (Toleranz: "+t+")");
		t = 19.99;
		equ = a.epsilonEquals(b, t);
		System.out.println("Ergebnis a==b: "+ equ + " (Toleranz: "+t+")");
		t = 20;
		equ = a.epsilonEquals(b, t);
		System.out.println("Ergebnis a==b: "+ equ + " (Toleranz: "+t+")");
		
		// Vektoraddition 
		// Achtung: das Ergebnis berschreibt den rsprnglichen Vektor
		Tuple3d sum = new Vector3d(a);
		sum.add(b);
		System.out.println("Ergebnis a + b:  "+sum);
		
		// Vektorsubtraktion 
		// Achtung: das Ergebnis berschreibt den rsprnglichen Vektor
		Tuple3d dif = new Vector3d(a);
		dif.sub(b);
		System.out.println("Ergebnis a - b:  "+dif);
		
		// Richtungsumkehr von Vektor a
		// Achtung: das Ergebnis berschreibt den rsprnglichen Vektor
		Tuple3d neg = new Vector3d(a);
		neg.negate();
		System.out.println("Ergebnis -a:     "+neg);
		
		
		// Methoden der Kindklasse Vector3d
		
		// Kreuzmultiplikation von zwei Vektoren 
		// Achtung: das Ergebnis berschreibt den rsprnglichen Vektor
		Vector3d xmul = new Vector3d();
		xmul.cross((Vector3d)a, (Vector3d)b);
		System.out.println("Ergebnis a X b:  "+xmul);
		xmul = new Vector3d();
		xmul.cross((Vector3d)b, (Vector3d)a);
		System.out.println("Ergebnis b X a:  "+xmul);

		// Skalarmultiplikation von zwei Vektoren 
		double smul = ((Vector3d)a).dot((Vector3d)b);
		System.out.println("Ergebnis a * b:  "+smul);
		
		// Aufgespannter Winkel zwischen a und b
		double ang = ((Vector3d)a).angle((Vector3d)b);
		ang = Math.toDegrees(ang);
		System.out.println("Ergebnis w(a,b): "+ang);
		
		// Lnge von Vector a
		double len = ((Vector3d)a).length();
		System.out.println("Ergebnis len(a): "+len);
		
		// Normalisierung des Vektors a
		// Achtung: das Ergebnis berschreibt den rsprnglichen Vektor
		Vector3d norm = new Vector3d(a);
		norm.normalize();
		System.out.println("Ergebnis nrm(a): "+norm);
		
		// Methoden der Kindklasse Point3d
		
		// Abstand zwischen a und b
		// Vektoren in Punkte wandeln
		Point3d ap = new Point3d(a);
		Point3d bp = new Point3d(b);
		double dist = ap.distance(bp);
		System.out.println("Ergebnis l(a,b): "+dist);
		
	}	// Ende Methode main()

}	// Ende Demo-Programm
